<?php
/**
 * Announcement Bar
 *
 * @package   Announcement_Bar
 * @author    Themify
 * @link      http://themify.me
 * @copyright 2013 Themify
 */

/**
 * Class Announcement_Options
 * @package Announcement_Bar
 */
 class Announcement_Options {

 	function __construct(){
		add_action( 'admin_menu', array( $this, 'add_plugin_page' ) );
 		add_action( 'admin_init', array( $this, 'page_init' ) );

 		add_action( 'admin_enqueue_scripts', array( $this, 'load_enqueue_scripts' ) );
 		add_filter( 'themify_top_pages', array( $this, 'queue_top_pages' ) );
 		add_filter( 'themify_pagenow', array( $this, 'queue_pagenow' ) );
 	}

 	function add_plugin_page(){
 		add_menu_page( 'Announcement Bar', 'Announcement Bar', 'manage_options', 'announcement-bar', array( $this, 'create_admin_page'), plugins_url( 'announcement-bar/themify/img/favicon.png' ) );
 	}

 	function create_admin_page(){
 		if ( ! current_user_can( 'manage_options' ) ) 
 			wp_die( __('You do not have sufficient permissions to access this page.', 'announcement-bar' ) );
 		
 		$options = Announcement_Model::get_plugin_settings();
 		$option_name = Announcement_Model::$option_name;
 		$default_content_teaser_length = Announcement_Model::get_default_option( 'ab_content_teaser_length' );

 		if ( isset( $_GET['action'] ) ) {
			$action = 'upgrade';
			announcement_bar_updater();
		}

		announcement_bar_flush_cache();
 		include( sprintf( "%s/views/settings.php", plugin_dir_path( dirname( __FILE__ ) ) ) );
 	}

 	function page_init(){
 		$option_name = Announcement_Model::$option_name;
 		register_setting( 'announcement_bar_group', $option_name );
 	}

 	function load_enqueue_scripts( $page ) {
 		global $wp_scripts, $typenow;
 		$types = themify_post_types();

 		if ( in_array( $typenow, $types ) || 'toplevel_page_announcement-bar' == $page ) {
 			wp_enqueue_script( 'jquery-ui-sortable' );

 			$ui = $wp_scripts->query('jquery-ui-core');
			$protocol = is_ssl() ? 'https': 'http';
			$url = "$protocol://ajax.googleapis.com/ajax/libs/jqueryui/{$ui->ver}/themes/smoothness/jquery-ui.css";
			wp_enqueue_style('jquery-ui-smoothness', $url, false, null);

 			if ( defined( 'THEMIFY_URI' ) ) {
 				wp_enqueue_style( 'themify-datetimepicker-css', THEMIFY_URI . '/css/jquery-ui-timepicker.css', array(), false );
				wp_enqueue_script( 'themify-datetimepicker-js', THEMIFY_URI . '/js/jquery-ui-timepicker.js', array( 'jquery', 'jquery-ui-datepicker'/*, 'jquery-ui-slider'*/ ), false, true );
 			} else {
 				wp_enqueue_style( 'themify-datetimepicker-css', ANNOUNCEMENT_BAR_PLUGIN_URL . '/themify/css/jquery-ui-timepicker.css', array(), false );
				wp_enqueue_script( 'themify-datetimepicker-js', ANNOUNCEMENT_BAR_PLUGIN_URL . '/themify/js/jquery-ui-timepicker.js', array( 'jquery', 'jquery-ui-datepicker'/*, 'jquery-ui-slider'*/ ), false, true );
 			}

 			if ( 'toplevel_page_announcement-bar' == $page ) {
 				wp_enqueue_script( 'announcement-bar-admin-scripts', ANNOUNCEMENT_BAR_PLUGIN_URL . '/js/admin-scripts.js', array('jquery'), false, true );
 			} else {
 				add_action( 'admin_footer', array( $this, 'hack_themify_field_js' ) );
 			}
 			
 		}
 	}

 	function queue_top_pages( $pages ) {
 		array_push( $pages, 'toplevel_page_announcement-bar' );
 		return $pages;
 	}

 	function queue_pagenow( $pagenows ) {
 		array_push( $pagenows, 'announcement-bar' );
 		return $pagenows;
 	}

 	public function hack_themify_field_js() {
 		echo "<script type=\"text/javascript\">jQuery(document).ready(function($){ $('#announcement_bar-default').closest('.themify_field_row').show(); })</script>";
 	}
 }